<?php

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/
Route::group(['prefix' => 'roulette', 'middleware' => 'auth'], function() {
    Route::post('/addBet', 'RouletteController@addBet');
    Route::post('/getBet', 'RouletteController@getBet');
    Route::get('/history', 'RouletteController@history');
});

Route::get('/king', ['as' => 'pvp', 'uses' => 'FlipController@index']);
Route::get('/double', ['as' => 'index', 'uses' => 'RouletteController@index']);
Route::get('/', ['as' => 'jackpot', 'uses' => 'JackpotController@jackpot']);

Route::post('/crash/addBet', 'CrashController@addBet');

Route::get('/result', 'PayController@result');
Route::get('/success', 'PayController@success');
Route::get('/fail', 'PayController@fail');

		Route::get('/admin/promo', ['as' => 'adminPromo', 'uses' => 'AdminController@promo']);
    Route::post('/admin/userSave', 'AdminController@userSave');
    Route::post('/admin/promoNew', 'AdminController@promoNew');
    Route::post('/admin/promoSave', 'AdminController@promoSave');
    Route::get('/admin/promoDelete/{id}', 'AdminController@promoDelete');

Route::group(['prefix' => '/auth'], function () {
    Route::get('/{provider}', 'AuthController@login');
    Route::get('/callback/{provider}', 'AuthController@callback');
});

Route::group(['middleware' => 'auth'], function () {
    /* Для авторизованных */
    Route::get('/logout', ['as' => 'logout', 'uses' => 'AuthController@logout']);
    Route::get('/j/history/', ['as' => 'historyAll', 'uses' => 'JackpotController@historyAll']);
    Route::get('/d/history', ['as' => 'doubleHistory', 'uses' => 'RouletteController@history']);
    Route::get('/payment/in', ['as' => 'payin', 'uses' => 'PayController@payin']);
    Route::get('/payment/out', ['as' => 'payout', 'uses' => 'PayController@payout']);
    Route::get('/payment/send', ['as' => 'paysend', 'uses' => 'PayController@paysend']);
    Route::get('/payment/history', ['as' => 'payhistory', 'uses' => 'PayController@payhistory']);
    Route::get('/pay', ['as' => 'pay', 'uses' => 'PayController@pay']);
    Route::get('/referral', ['as' => 'referral', 'uses' => 'PageController@referral']);
    Route::get('/youtube', ['as' => 'youtube', 'uses' => 'PageController@youtube']);
    Route::get('/bonus', ['as' => 'bonus', 'uses' => 'PageController@bonus']);
    Route::get('/faq', ['as' => 'faq', 'uses' => 'PageController@faq']);
 Route::get('/rules', ['as' => 'rules', 'uses' => 'PageController@rules']);
    Route::get('/terms', ['as' => 'terms', 'uses' => 'PageController@terms']);
    Route::post('/withdraw', 'PayController@withdraw');
    Route::post('/newBet', 'JackpotController@newBet');
    Route::post('/chat', 'ChatController@add_message');
    Route::post('/payment/send/create', 'PayController@sendCreate');
    Route::post('/referral/activate', 'PageController@refActivate');
    Route::post('/referral/getMoney', 'PageController@getMoney');
    Route::post('/bonus/getBonus', 'PageController@getBonus');
    Route::post('/bonus/takeBonusVK', 'PageController@takeBonusVK');
    # flip
	Route::post('/flip/newGame', 'FlipController@createRoom'); 
	Route::post('/flip/joinRoom', 'FlipController@joinRoom'); 
});

Route::group(['middleware' => 'auth', 'middleware' => 'access:admin'], function () {
	Route::get('/admin', ['as' => 'admin', 'uses' => 'AdminController@index']);
	Route::get('/admin/botOn', ['as' => 'botOn', 'uses' => 'AdminController@botOn']);
	Route::get('/admin/botOff', ['as' => 'botOff', 'uses' => 'AdminController@botOff']);
	Route::get('/admin/users', ['as' => 'adminUsers', 'uses' => 'AdminController@users']);
	Route::get('/admin/user/{id}', ['as' => 'adminUser', 'uses' => 'AdminController@user']);
	Route::get('/admin/settings', ['as' => 'adminSettings', 'uses' => 'AdminController@settings']);
	Route::get('/admin/withdraw', ['as' => 'adminWithdraw', 'uses' => 'AdminController@withdraw']);

	Route::get('/admin/payments', ['as' => 'admin', 'uses' => 'AdminController@payments']);

	Route::get('/admin/podkrytka', ['as' => 'admin', 'uses' => 'AdminController@podkrytka']);

	Route::get('/admin/promo', ['as' => 'adminPromo', 'uses' => 'AdminController@promo']);
    Route::post('/admin/userSave', 'AdminController@userSave');
    Route::post('/admin/promoNew', 'AdminController@promoNew');
    Route::post('/admin/promoSave', 'AdminController@promoSave');
    Route::get('/admin/promoDelete/{id}', 'AdminController@promoDelete');
    Route::post('/admin/settingSave', 'AdminController@settingsSave');
    Route::get('/admin/withdraw/{id}', 'AdminController@withdrawSend');
    Route::get('/admin/return/{id}', 'AdminController@withdrawReturn');
    Route::post('/admin/gotDouble', 'RouletteController@gotDouble');
    Route::post('/admin/gotRoulette', 'JackpotController@gotRoulette');
});

Route::group(['prefix' => 'api', 'middleware' => 'secretKey'], function () {

    Route::group(['prefix' => 'roulette'], function() {
        Route::post('/getGame', 'RouletteController@getGame');
        Route::post('/updateStatus', 'RouletteController@updateStatus');
        Route::post('/getSlider', 'RouletteController@getSlider');
        Route::post('/newGame', 'RouletteController@newGame');
    });

    Route::group(['prefix' => 'crash'], function() {
        Route::post('/getGame', 'CrashController@getGame');
        Route::post('/updateStatus', 'CrashController@updateStatus');
        Route::post('/slider', 'CrashController@getSlider');
        Route::post('/newGame', 'CrashController@newGame');
        Route::post('/withdraw', 'CrashController@withdraw');
    });

    Route::post('/jackpot/getSlider', 'JackpotController@getSlider');
    Route::post('/jackpot/newGame', 'JackpotController@newGame');
    Route::post('/jackpot/getStatus', 'JackpotController@getStatus');
    Route::post('/jackpot/addOffer', 'JackpotController@addOffer');
    Route::post('/jackpot/setOfferStatus', 'JackpotController@setOfferStatus');
    Route::post('/jackpot/setStatus', 'JackpotController@setStatus');
    
    Route::post('/getMerchBalance', 'AdminController@getMerchBalance');
});